@@ -18,6 +18,11 @@ chordDiagram = function(x, ...) {
 	if(inherits(x, "matrix")) {
 		chordDiagramFromMatrix(x, ...)
 	} else if(inherits(x, "data.frame")) {
+		if(ncol(x) > 3) {
+			if(all(sapply(x, inherits, "numeric"))) {
+				warning("It seems your input data is an adjacency matrix, maybe you need to convert it to 'matrix' explicitely.")
+			}
+		}
 		chordDiagramFromDataFrame(x, ...)
 	} else {
 		stop("`x` can only be a matrix or a data frame.")
