@@ -230,7 +230,8 @@ colorRamp2 = function(breaks, colors, transparency = 0, space = "LAB") {
   }
   
   colors = t(col2rgb(colors)/255)
-  
+
+  attr = list(breaks = breaks, colors = colors, transparency = transparency, space = space)
 
   if(space == "LUV") {
     i = which(apply(colors, 1, function(x) all(x == 0)))
@@ -241,7 +242,7 @@ colorRamp2 = function(breaks, colors, transparency = 0, space = "LAB") {
   transparency_str = sprintf("%X", round(transparency*255))
   if(nchar(transparency_str) == 1) transparency_str = paste0("0", transparency_str)
   
-  fun = function(x = NULL) {
+  fun = function(x = NULL, return_rgb = FALSE, max_value = 1) {
     if(is.null(x)) {
       stop("Please specify `x`\n")
     }
@@ -259,10 +260,14 @@ colorRamp2 = function(breaks, colors, transparency = 0, space = "LAB") {
     }
     res_col = paste(res_col, transparency_str[1], sep = "")
     attributes(res_col) = att
+    
+    if(return_rgb) {
+      res_col = t(col2rgb(as.vector(res_col), alpha = TRUE)/255)
+    }
     return(res_col)
   }
   
-  attr(fun, "breaks") = breaks
+  attributes(fun) = attr
   return(fun)
 }
 
@@ -279,8 +284,8 @@ colorRamp2 = function(breaks, colors, transparency = 0, space = "LAB") {
 # rgb2 vector with 3 elements
 .get_color = function(x, break1, break2, col1, col2, space) {
 
-  col1 = coords(as(RGB(col1[1], col1[2], col1[3]), space))
-  col2 = coords(as(RGB(col2[1], col2[2], col2[3]), space))
+  col1 = coords(as(sRGB(col1[1], col1[2], col1[3]), space))
+  col2 = coords(as(sRGB(col2[1], col2[2], col2[3]), space))
 
   res_col = matrix(ncol = 3, nrow = length(x))
   for(i in seq_along(x)) {
@@ -289,11 +294,11 @@ colorRamp2 = function(breaks, colors, transparency = 0, space = "LAB") {
   }
   
   res_col = eval(parse(text = paste0(space, "(res_col)")))
-  res_col = coords(as(res_col, "RGB"))
+  res_col = coords(as(res_col, "sRGB"))
   res_col[, 1] = .restrict_in(res_col[,1], 0, 1)
   res_col[, 2] = .restrict_in(res_col[,2], 0, 1)
   res_col[, 3] = .restrict_in(res_col[,3], 0, 1)
-  hex(RGB(res_col))
+  hex(sRGB(res_col))
 }
 
 # will be considered in the future
@@ -399,3 +404,79 @@ adjacencyList2Matrix = function(lt, square = FALSE) {
 	return(mat)
 }
 
+
+
+# == title
+# Transform colors to values
+#
+# == param
+# -r red channel in `colorspace::sRGB` color space, value should be between 0 and 1. It can also be a three-column matrix.
+# -g green channel in `colorspace::sRGB` color space, value should be between 0 and 1.
+# -b blue channel in `colorspace::sRGB` color space, value should be between 0 and 1.
+# -col_fun the color mapping function generated by `colorRamp2`.
+#
+# == details
+# `colorRamp2` transforms values to colors and this function does the reversed job.
+# Note for some color spaces, it cannot transform back to the original value perfectly.
+#
+# == author
+# Zuguang Gu <z.gu@dkfz.de>
+#
+# == example
+# x = seq(0, 1, length = 11)
+# col_fun = colorRamp2(c(0, 0.5, 1), c("blue", "white", "red"))
+# col = col_fun(x)
+# col2value(col, col_fun = col_fun)
+#
+# col_fun = colorRamp2(c(0, 0.5, 1), c("blue", "white", "red"), space = "sRGB")
+# col = col_fun(x)
+# col2value(col, col_fun = col_fun)
+col2value = function(r, g, b, col_fun) {
+  if(missing(g) && missing(b)) {
+    if(is.character(r)) {
+      rgb = col2rgb(r)/255
+      r = rgb[1, ]
+      g = rgb[2, ]
+      b = rgb[3, ]
+    } else {
+      if(is.list(r)) {
+        g = r[[2]]
+        b = r[[3]]
+        r = r[[1]]
+      } else if(is.data.frame(r) || is.matrix(r)) {
+        g = r[, 2]
+        b = r[, 3]
+        r = r[, 1]
+      }
+    }
+  }
+
+  breaks = attr(col_fun, "breaks")
+  colors = attr(col_fun, "colors")
+  space = attr(col_fun, "space")
+
+  n = length(r)
+
+  ## convert all colors to the specified space
+  m = coords(as(sRGB(r, g, b), space))
+  breaks_m = coords(as(sRGB(colors), space))
+  n_breaks = length(breaks)
+
+  # interpolation for the three channels seperatedly
+  v = numeric()
+  for(i in seq_len(nrow(m))) {
+    l1 = (breaks_m[-n_breaks, 1] <= m[i, 1] & breaks_m[-1, 1] >= m[i, 1]) | (breaks_m[-n_breaks, 1] >= m[i, 1] & breaks_m[-1, 1] <= m[i, 1])
+    l2 = (breaks_m[-n_breaks, 2] <= m[i, 2] & breaks_m[-1, 2] >= m[i, 2]) | (breaks_m[-n_breaks, 2] >= m[i, 2] & breaks_m[-1, 2] <= m[i, 2])
+    l3 = (breaks_m[-n_breaks, 3] <= m[i, 3] & breaks_m[-1, 3] >= m[i, 3]) | (breaks_m[-n_breaks, 3] >= m[i, 3] & breaks_m[-1, 3] <= m[i, 3])
+    k = which(l1 & l2 & l3)[1]
+    v1 = (breaks[k] - breaks[k+1])/(breaks_m[k, 1] - breaks_m[k+1, 1]) * (m[i, 1] - breaks_m[k, 1]) + breaks[k]
+    v2 = (breaks[k] - breaks[k+1])/(breaks_m[k, 2] - breaks_m[k+1, 2]) * (m[i, 2] - breaks_m[k, 2]) + breaks[k]
+    v3 = (breaks[k] - breaks[k+1])/(breaks_m[k, 3] - breaks_m[k+1, 3]) * (m[i, 3] - breaks_m[k, 3]) + breaks[k]
+    vv = c(v1, v2, v3)
+    v[i] = mean(vv, na.rm = TRUE)
+  }
+  return(v)
+}
+
+
+
